@@ -1,17 +1,24 @@
 import ipaddress
 
+from math import ceil
+from datetime import datetime
+
 from django.core.validators import MinValueValidator, MaxValueValidator
+from django.core.exceptions import ObjectDoesNotExist
 from django.db import models, transaction
-from django.db.models import Q
+from django.db.models import Q, Max, ExpressionWrapper, BooleanField
 from django.db.models.functions import Length
 from django.urls import reverse
 
+from django.db.models.signals import m2m_changed
+from django.dispatch import receiver
+
 from extras.utils import extras_features
 from netbox.models import PrimaryModel, TaggableManager
 from utilities.querysets import RestrictedQuerySet
 
 
-@extras_features("custom_fields", "custom_links", "export_templates", "webhooks")
+@extras_features("custom_links", "export_templates", "webhooks")
 class NameServer(PrimaryModel):
     name = models.CharField(
         unique=True,
@@ -36,21 +43,44 @@ class NameServer(PrimaryModel):
         return reverse("plugins:netbox_dns:nameserver", kwargs={"pk": self.pk})
 
 
-@extras_features("custom_fields", "custom_links", "export_templates", "webhooks")
+class ZoneManager(models.Manager.from_queryset(RestrictedQuerySet)):
+    """Special Manager for zones providing the activity status annotation"""
+
+    def get_queryset(self):
+        return (
+            super(ZoneManager, self)
+            .get_queryset()
+            .annotate(
+                active=ExpressionWrapper(
+                    Q(status__in=Zone.ACTIVE_STATUS_LIST), output_field=BooleanField()
+                )
+            )
+        )
+
+
+@extras_features("custom_links", "export_templates", "webhooks")
 class Zone(PrimaryModel):
     STATUS_ACTIVE = "active"
-    STATUS_PASSIVE = "passive"
+    STATUS_RESERVED = "reserved"
+    STATUS_DEPRECATED = "deprecated"
+    STATUS_PARKED = "parked"
 
     CHOICES = (
         (STATUS_ACTIVE, "Active"),
-        (STATUS_PASSIVE, "Passive"),
+        (STATUS_RESERVED, "Reserved"),
+        (STATUS_DEPRECATED, "Deprecated"),
+        (STATUS_PARKED, "Parked"),
     )
 
     CSS_CLASSES = {
-        STATUS_PASSIVE: "danger",
-        STATUS_ACTIVE: "success",
+        STATUS_ACTIVE: "primary",
+        STATUS_RESERVED: "info",
+        STATUS_DEPRECATED: "danger",
+        STATUS_PARKED: "warning",
     }
 
+    ACTIVE_STATUS_LIST = (STATUS_ACTIVE,)
+
     name = models.CharField(
         unique=True,
         max_length=255,
@@ -95,11 +125,11 @@ class Zone(PrimaryModel):
         null=False,
         verbose_name="SOA RName",
     )
-    soa_serial = models.PositiveIntegerField(
-        blank=False,
-        null=False,
+    soa_serial = models.BigIntegerField(
+        blank=True,
+        null=True,
         verbose_name="SOA Serial",
-        validators=[MinValueValidator(1), MaxValueValidator(2147483647)],
+        validators=[MinValueValidator(1), MaxValueValidator(4294967295)],
     )
     soa_refresh = models.PositiveIntegerField(
         blank=False,
@@ -125,8 +155,13 @@ class Zone(PrimaryModel):
         verbose_name="SOA Minimum TTL",
         validators=[MinValueValidator(1)],
     )
+    soa_serial_auto = models.BooleanField(
+        verbose_name="Generate SOA Serial",
+        help_text="Automatically generate the SOA Serial field",
+        default=True,
+    )
 
-    objects = RestrictedQuerySet.as_manager()
+    objects = ZoneManager()
 
     clone_fields = [
         "name",
@@ -186,6 +221,85 @@ class Zone(PrimaryModel):
                 managed=True,
             )
 
+    def update_ns_records(self, nameservers):
+        ns_name = "@"
+        ns_ttl = self.default_ttl
+
+        delete_ns = self.record_set.filter(type=Record.NS, managed=True).exclude(
+            value__in=nameservers
+        )
+        for record in delete_ns:
+            record.delete()
+
+        for ns in nameservers:
+            Record.raw_objects.update_or_create(
+                zone_id=self.pk,
+                type=Record.NS,
+                name=ns_name,
+                ttl=ns_ttl,
+                value=ns,
+                managed=True,
+            )
+
+    def check_nameservers(self):
+        nameservers = self.nameservers.all()
+
+        ns_warnings = []
+        ns_errors = []
+
+        if not nameservers:
+            ns_errors.append(f"No nameservers are configured for zone {self.name}")
+
+        for nameserver in nameservers:
+            ns_domain = ".".join(nameserver.name.split(".")[1:])
+            if not ns_domain:
+                continue
+
+            try:
+                ns_zone = Zone.objects.get(name=ns_domain)
+            except ObjectDoesNotExist:
+                continue
+
+            ns_name = nameserver.name.split(".")[0]
+            address_records = Record.objects.filter(
+                Q(zone=ns_zone),
+                Q(Q(name=f"{nameserver.name}.") | Q(name=ns_name)),
+                Q(Q(type=Record.A) | Q(type=Record.AAAA)),
+            )
+
+            if not address_records:
+                ns_warnings.append(
+                    f"Nameserver {nameserver.name} does not have an address record in zone {ns_zone.name}"
+                )
+
+        return ns_warnings, ns_errors
+
+    def get_auto_serial(self):
+        records = Record.objects.filter(zone=self).exclude(type=Record.SOA)
+        if records:
+            soa_serial = (
+                records.aggregate(Max("last_updated"))
+                .get("last_updated__max")
+                .timestamp()
+            )
+        else:
+            soa_serial = ceil(datetime.now().timestamp())
+
+        if self.last_updated:
+            soa_serial = ceil(max(soa_serial, self.last_updated.timestamp()))
+
+        return soa_serial
+
+    def update_serial(self):
+        self.last_updated = datetime.now()
+        self.save()
+
+    def parent_zones(self):
+        zone_fields = self.name.split(".")
+        return [
+            f'{".".join(zone_fields[length:])}' for length in range(1, len(zone_fields))
+        ]
+
     def save(self, *args, **kwargs):
         new_zone = self.pk is None
         if not new_zone:
@@ -193,12 +307,18 @@ class Zone(PrimaryModel):
         else:
             renamed_zone = False
 
+        if self.soa_serial_auto:
+            self.soa_serial = self.get_auto_serial()
+
         super().save(*args, **kwargs)
 
         if (new_zone or renamed_zone) and self.name.endswith(".arpa"):
             address_records = Record.objects.filter(
-                type__in=(Record.A, Record.AAAA)
-            ).exclude(disable_ptr=True)
+                Q(ptr_record__isnull=True)
+                | Q(ptr_record__zone__name__in=self.parent_zones()),
+                type__in=(Record.A, Record.AAAA),
+                disable_ptr=False,
+            )
             for record in address_records:
                 record.update_ptr_record()
 
@@ -211,20 +331,60 @@ class Zone(PrimaryModel):
     def delete(self, *args, **kwargs):
         with transaction.atomic():
             address_records = list(self.record_set.filter(ptr_record__isnull=False))
-            ptr_records = self.record_set.filter(address_record__isnull=False)
-
             for record in address_records:
                 record.ptr_record.delete()
 
-            address_records.extend(record.address_record for record in ptr_records)
+            ptr_records = self.record_set.filter(address_record__isnull=False)
+            update_records = [
+                record.pk
+                for record in Record.objects.filter(ptr_record__in=ptr_records)
+            ]
 
             super().delete(*args, **kwargs)
 
-        for record in address_records:
+        for record in Record.objects.filter(pk__in=update_records):
             record.update_ptr_record()
 
 
-@extras_features("custom_fields", "custom_links", "export_templates", "webhooks")
+@receiver(m2m_changed, sender=Zone.nameservers.through)
+def update_ns_records(**kwargs):
+    if kwargs.get("action") not in ["post_add", "post_remove"]:
+        return
+
+    zone = kwargs.get("instance")
+    nameservers = zone.nameservers.all()
+
+    new_nameservers = [f'{ns.name.rstrip(".")}.' for ns in nameservers]
+
+    zone.update_ns_records(new_nameservers)
+
+
+class RecordManager(models.Manager.from_queryset(RestrictedQuerySet)):
+    """Special Manager for records providing the activity status annotation"""
+
+    def get_queryset(self):
+        return (
+            super(RecordManager, self)
+            .get_queryset()
+            .annotate(
+                active=ExpressionWrapper(
+                    Q(
+                        Q(zone__status__in=Zone.ACTIVE_STATUS_LIST)
+                        & Q(
+                            Q(address_record__isnull=True)
+                            | Q(
+                                address_record__zone__status__in=Zone.ACTIVE_STATUS_LIST
+                            )
+                        )
+                    ),
+                    output_field=BooleanField(),
+                )
+            )
+        )
+        return queryset
+
+
+@extras_features("custom_links", "export_templates", "webhooks")
 class Record(PrimaryModel):
     A = "A"
     AAAA = "AAAA"
@@ -327,7 +487,8 @@ class Record(PrimaryModel):
         default=False,
     )
 
-    objects = RestrictedQuerySet.as_manager()
+    objects = RecordManager()
+    raw_objects = RestrictedQuerySet.as_manager()
 
     clone_fields = ["zone", "type", "name", "value", "ttl", "disable_ptr"]
 
@@ -427,8 +588,16 @@ class Record(PrimaryModel):
 
         super().save(*args, **kwargs)
 
+        zone = self.zone
+        if self.type != self.SOA and zone.soa_serial_auto:
+            zone.update_serial()
+
     def delete(self, *args, **kwargs):
         if self.ptr_record:
             self.ptr_record.delete()
 
         super().delete(*args, **kwargs)
+
+        zone = self.zone
+        if zone.soa_serial_auto:
+            zone.update_serial()
